from flask import Flask, request, jsonify, render_template, url_for
from flask_socketio import SocketIO
import requests
import os
import yaml
import logging
import datetime
from dotenv import load_dotenv
import time  # Importing the missing time module
import webbrowser  # To open the browser

# Importing custom modules
from data_fetcher import fetch_stock_data
from strategy import generate_signals, train_model, load_model, generate_signals_with_model
from backtester import execute_backtest, analyze_performance
from visualizer import plot_performance
from notifier import send_email
from utils import fetch_balance

# Load environment variables
load_dotenv()

# Define the Flask app
app = Flask(__name__, static_folder='static', template_folder='templates')
app.config['SECRET_KEY'] = os.getenv('SECRET_KEY', 'default_secret_key')
socketio = SocketIO(app)

# 보안 설정 추가
app.config['SESSION_COOKIE_SECURE'] = True
app.config['REMEMBER_COOKIE_SECURE'] = True
app.config['SESSION_COOKIE_HTTPONLY'] = True
app.config['REMEMBER_COOKIE_HTTPONLY'] = True
app.config['SESSION_PROTECTION'] = 'strong'

# Load configuration from YAML file
with open('config.yaml', 'r', encoding='utf-8') as file:
    config = yaml.safe_load(file)

# Token caching
access_token = None
token_expiry = None

def get_access_token():
    global access_token, token_expiry

    # Check if token is still valid
    if access_token and token_expiry and token_expiry > datetime.datetime.now():
        return access_token

    # Fetch new token
    url = f"{config['api_config']['access_token']['domain']}{config['api_config']['access_token']['url']}"
    headers = {
        'content-type': 'application/json; charset=utf-8'
    }
    grant_type = os.getenv('GRANT_TYPE') or config['api_config']['access_token']['grant_type']
    data = {
        'grant_type': grant_type,
        'client_id': os.getenv('CLIENT_ID') or config['api_config']['access_token']['client_id'],
        'client_secret': os.getenv('CLIENT_SECRET') or config['api_config']['access_token']['client_secret']
    }

    response = requests.post(url, headers=headers, json=data)
    response_data = response.json()
    access_token = response_data['access_token']
    expires_in = response_data['expires_in']
    token_expiry = datetime.datetime.now() + datetime.timedelta(seconds=expires_in)
    return access_token

@app.route('/')
def index():
    data = fetch_stock_data()
    return render_template('index.html', data=data)

def background_thread():
    while True:
        data = fetch_stock_data()
        socketio.emit('updateData', data)
        time.sleep(10)  # Fetch data every 10 seconds

@socketio.on('connect')
def test_connect():
    socketio.start_background_task(target=background_thread)

# Function to open the web browser
def open_browser():
    webbrowser.open_new('http://127.0.0.1:5000/')

if __name__ == '__main__':
    # Open the web browser before starting the server
    open_browser()
    socketio.run(app, debug=True)
